/* Include Tutorial files */
require("menu.nut");
require("tutorial_step.nut");

/* Import SuperLib for GameScript */
require("NoGoSuperLib/main.nut");
Result <- SuperLib.Result;
Log <- SuperLib.Log;
Helper <- SuperLib.Helper;
ScoreList <- SuperLib.ScoreList;
Tile <- SuperLib.Tile;
Direction <- SuperLib.Direction;
Town <- SuperLib.Town;
Industry <- SuperLib.Industry;

/* Globals */
g_menu <- null;


class TestGame extends GSController 
{
	_chapter_steps = null;
	_current_step = null;

	_chapter_storage = null;

	_load_data = null;

	constructor()
	{
	}
}

/*
 * -------- Tutorial Engine --------------------------
 */
function TestGame::Start()
{
	// Any GameScript init code goes here
	this.Init();

	Log.Info("Tutorial setup done", Log.LVL_INFO);

	// Wait for the game to start
	this.Sleep(1);

	// Only when the game has started, the human company exist
	local company_mode = GSCompanyMode(0);

	Log.Info("Tutorial starts now", Log.LVL_INFO);

	while (true) {
		this.HandleEvents();
		this.RunTutorial();
		this.Sleep(10);
	}
}

function TestGame::Init()
{
	g_menu = Menu();

	this.InitBeforeNewChapter();

	local loaded_state = false;
	if(this._load_data != null)
	{
		if(this._load_data.chapter == "bus")
		{
			// Set current step to the step it had when the game was saved
			this._current_step = this._load_data.step; 

			// Load the bus chapter
			this.LoadBusChapter();

			loaded_state = true;
		}
	}

	if(!loaded_state)
	{
		// No tutorial state was loaded from save
		// => Start from the beginning (in future: possible show a menu of chapters)
		this.LoadBusChapter();
	}
}

function TestGame::InitBeforeNewChapter()
{
	this._chapter_steps = [];
	this._current_step = -1;
	this._chapter_storage = {}; // Table which all chapter steps can access to store data
}
function TestGame::AddStep( new_step )
{
	this._chapter_steps.append(new_step);
	new_step.SetStorageTable(this._chapter_storage);
}

function TestGame::HandleEvents()
{
}

function TestGame::RunTutorial()
{
	// Is tutorial already done?
	if(this._current_step >= this._chapter_steps.len())
		return;

	// Is last step done?
	if(this._current_step < 0 || this._chapter_steps[this._current_step].IsDone())
	{
		this._current_step++;

		// reached end of tutorial?
		if(this._current_step >= this._chapter_steps.len())
			return;

		// Execute next step
		this._chapter_steps[this._current_step].Execute();
	}
}

function TestGame::Save()
{
	return { 
		chapter = "bus",
  		step = this._current_step
	};
}

function TestGame::Load(tbl, version)
{
	// Store a copy of the table from the save game
	this._load_data = { 
		chapter = tbl.chapter,
		step = tbl.step
	};
}

/*
 * -------- Chapters ---------------------------------
 */
function TestGame::LoadBusChapter()
{
	this.AddStep(MessageStep("Hello and welcome to the bus tutorial"));
	this.AddStep(MessageStep("Another message"));
	this.AddStep(MessageStep("Thanks for taking this tutorial"));
}

