
/*
 * Base class for all tutorial step classes
 */
class TutorialStep
{
	/*
	 * Pointer to a table in which the chapter can store
	 * data between the steps.
	 */
	_storage = null;

	constructor()
	{
	}

	/*
	 * This function should execute a tutorial step
	 * Eg. display a message, highlight a button and/or
	 * moving the screen.
	 * A Step may do just a single task or many things,
	 * but it should return when it is done and not
	 * wait for user feedback.
	 */
	function Execute() // override this
	{
	}

	/*
	 * After a step has been executed, the tutorial engine
	 * will poll this function to see if the step is done or not. 
	 * 
	 * This function should return true when the tutorial should
	 * progress to the next step. If you return false,
	 * the engine will check events etc. and then poll again.
	 */
	function IsDone() // override this
	{
		return true;
	}

	// don't override this function:
	function SetStorageTable(table)
	{
		this._storage = table;
	}
}

/*
 * A message step displays a message and wait for the user to
 * click on the continue button.
 */
class MessageStep extends TutorialStep
{
	_message = null;

	constructor(message)
	{
		this._message = message;
	}

	function Execute();
	function IsDone();
}

function MessageStep::Execute()
{
	g_menu.Open(this._message, ["continue"]);
}

function MessageStep::IsDone()
{
	// This tutorial step is done when the user has closed the menu
	local btn = g_menu.CheckInput();
	return btn != MENU_OPEN;
}


